@@ -4,11 +4,11 @@
  *
  * Authors: Miguel de Icaza (GNOME terminal)
  *          Erik Troan      (various configuration enhancements)
- *          Michael Zucchi  (zvt widget, font code).
+ *          Michael Zucchi  (zvt widget, various updates and enhancements)
  *
  * Other contributors: George Lebl, Jeff Garzik, Jay Painter,
  * Christopher Blizzard, Jens Lautenbacher, Tom Tromey, Tristan Tarant,
- * Jonathan Blandford and Nat Friedman
+ * Jonathan Blandford, Cody Russell, and Nat Friedman
  */
 #include <config.h>
 #include <unistd.h>
@@ -37,9 +37,6 @@ char **env;
 /* Initial geometry */
 char *geometry = 0;
 
-/* By default record utmp logins */
-gboolean update_records = ZVT_TERM_DO_UTMP_LOG | ZVT_TERM_DO_WTMP_LOG;
-
 /* is there pixmap compiled into zvt */
 static gboolean zvt_pixmap_support = FALSE;
 
@@ -56,6 +53,13 @@ enum color_set_enum {
 	COLORS_CUSTOM
 };
 
+enum palette_enum {
+	PALETTE_LINUX,
+	PALETTE_XTERM,
+	PALETTE_RXVT,
+	PALETTE_CUSTOM
+};
+
 enum scrollbar_position_enum {
 	SCROLLBAR_LEFT   = 0,
 	SCROLLBAR_RIGHT  = 1,
@@ -78,15 +82,15 @@ struct terminal_config {
 #ifdef ZVT_BACKGROUND_SCROLL
 	int scroll_background:1; 		/* background will scroll */
 #endif
-	int color_type; 			/* The color mode */
+	enum palette_enum color_type; 			/* The color mode */
 	enum color_set_enum color_set;
 	char *font; 				/* Font used by the terminals */
 	int scrollback; 			/* Number of scrollbacklines */
 	char *class;
 	enum scrollbar_position_enum scrollbar_position;
 	int invoke_as_login_shell; 		/* How to invoke the shell */
-	GdkColor user_fore, user_back; 		/* The custom colors */
-        const char *user_back_str, *user_fore_str;
+	int update_records;
+	const char *user_back_str, *user_fore_str;
 	int menubar_hidden; 			/* Whether to show the menubar */
 	int have_user_colors;			/* Only used for command line parsing */
 	int transparent;
@@ -96,6 +100,7 @@ struct terminal_config {
         char *window_title;                     /* the window title */
 	char *wordclass;			/* select-by-word character class */
 	char *termname;				/* TERM variable setting, store as TERM=xxx */
+	GdkColor palette[18];			/* the full palette */
 } ;
 
 /* Initial command */
@@ -134,9 +139,9 @@ typedef struct {
 	GtkWidget *scrollback_spin;
 	GtkWidget *class_box;
 	GtkWidget *fore_label;
-	GtkWidget *fore_cs;
 	GtkWidget *back_label;
-	GtkWidget *back_cs;
+	GtkWidget *palette_label;
+	GtkWidget *palette[18];
 	int changed;
 } preferences_t;
 
@@ -147,6 +152,16 @@ typedef struct {
  */
 #define POPUP_MENU_TOGGLE_INDEX_MENUBAR 2
 #define POPUP_MENU_TOGGLE_INDEX_SECURE  3
+/* eek, multi-conditaional for backward compatability *sigh* */
+#ifdef ZVT_TERM_MATCH_SUPPORT
+# ifdef HAVE_ZVT_TERM_RESET
+#  define POPUP_MENU_DYNAMIC_INDEX 6
+#  define POPUP_MENU_LAST_INDEX 7
+# else
+#  define POPUP_MENU_DYNAMIC_INDEX 4
+#  define POPUP_MENU_LAST_INDEX 5
+# endif
+#endif
 
 /*
  * Exported interfaces, for Gtk modules that hook
@@ -160,6 +175,9 @@ void paste_cmd            (GtkWidget *widget, ZvtTerm *term);
 void preferences_cmd      (GtkWidget *widget, ZvtTerm *term);
 void toggle_secure_keyboard_cmd (GtkWidget *w, ZvtTerm *term);
 
+static int popup_menu_cmd (ZvtTerm *term, GdkEventButton *event,
+			   GtkWidget *menu, GnomeUIInfo *uiinfo);
+
 GtkWidget *new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry);
 GtkWidget *new_terminal     (GtkWidget *widget, ZvtTerm *term);
 
@@ -319,29 +337,40 @@ gushort rxvt_blu[] = { 0x0000, 0x0000, 0x0000, 0x0000, 0xffff, 0xffff, 0xffff, 0
 		       0x0000, 0x0000, 0x0000, 0x0000, 0xffff, 0xffff, 0xffff, 0xffff,
 			0x0,    0x0 };
 
-/* These point to the current table */
-gushort *red, *blue, *green;
+/* point to the other tables */
+gushort *scheme_red[] = { linux_red, xterm_red, rxvt_red, rxvt_red };
+gushort *scheme_blue[] = { linux_blu, xterm_blu, rxvt_blu, rxvt_blu };
+gushort *scheme_green[] = { linux_grn, xterm_grn, rxvt_grn, rxvt_grn };
 
 static void
 set_color_scheme (ZvtTerm *term, struct terminal_config *color_cfg) 
 {
 	GdkColor c;
-	
+	gushort red[18],green[18],blue[18];
+	int i;
+	gushort *r, *b, *g;
+
 	switch (color_cfg->color_type){
-	case 0:
-		red = linux_red;
-		green = linux_grn;
-		blue = linux_blu;
+	default:		/* and 0 */
+		color_cfg->color_type = 0;
+	case PALETTE_LINUX:
+	case PALETTE_XTERM:
+	case PALETTE_RXVT:
+		r = scheme_red[color_cfg->color_type];
+		g = scheme_green[color_cfg->color_type];
+		b = scheme_blue[color_cfg->color_type];
+		for (i=0;i<18;i++) {
+			red[i] = r[i];
+			green[i] = g[i];
+			blue[i] = b[i];
+		}
 		break;
-	case 1:
-		red = xterm_red;
-		green = xterm_grn;
-		blue = xterm_blu;
-		break;
-	default:		/* and 2 */
-		red = rxvt_red;
-		green = rxvt_grn;
-		blue = rxvt_blu;
+	case PALETTE_CUSTOM:
+		for (i=0;i<18;i++) {
+			red[i] = color_cfg->palette[i].red;
+			green[i] = color_cfg->palette[i].green;
+			blue[i] = color_cfg->palette[i].blue;
+		}
 		break;
 	}
 
@@ -388,12 +417,11 @@ set_color_scheme (ZvtTerm *term, struct terminal_config *color_cfg)
 		
 		/* Custom foreground, custom background */
 	case COLORS_CUSTOM:
-		red   [16] = color_cfg->user_fore.red;
-		green [16] = color_cfg->user_fore.green;
-		blue  [16] = color_cfg->user_fore.blue;
-		red   [17] = color_cfg->user_back.red;
-		green [17] = color_cfg->user_back.green;
-		blue  [17] = color_cfg->user_back.blue;
+		for (i=16;i<18;i++) {
+			red[i] = color_cfg->palette[i].red;
+			green[i] = color_cfg->palette[i].green;
+			blue[i] = color_cfg->palette[i].blue;
+		}
 		break;
 	}
 	zvt_term_set_color_scheme (term, red, green, blue);
@@ -410,6 +438,9 @@ load_config (char *class)
 	char *fore_color = NULL;
 	char *back_color = NULL;
 	struct terminal_config *cfg = g_malloc (sizeof (*cfg));
+	int colour_count;
+	char **colours;
+	int i;
 
 	/* It's very odd that these are here */
 	cfg->font = NULL;
@@ -427,11 +458,15 @@ load_config (char *class)
 		cfg->scrollbar_position = SCROLLBAR_HIDDEN;
 	p = gnome_config_get_string ("color_scheme=linux");
 	if (strcasecmp (p, "linux") == 0)
-		cfg->color_type = 0;
+		cfg->color_type = PALETTE_LINUX;
 	else if (strcasecmp (p, "xterm") == 0)
-		cfg->color_type = 1;
+		cfg->color_type = PALETTE_XTERM;
+	else if (strcasecmp (p, "rxvt") == 0)
+		cfg->color_type = PALETTE_RXVT;
+	else if (strcasecmp (p, "custom") == 0)
+		cfg->color_type = PALETTE_CUSTOM;
 	else
-		cfg->color_type = 2;
+		cfg->color_type = PALETTE_LINUX;
 	cfg->bell      = gnome_config_get_bool ("bell_silenced=0");
 	cfg->blink     = gnome_config_get_bool ("blinking=0");
 	cfg->swap_keys = gnome_config_get_bool ("swap_del_and_backspace=0");
@@ -458,16 +493,32 @@ load_config (char *class)
 
 	cfg->termname = NULL;
 
+	cfg->update_records = ZVT_TERM_DO_UTMP_LOG|ZVT_TERM_DO_WTMP_LOG;
+
 	if (strcasecmp (fore_color, back_color) == 0)
 		/* don't let them set identical foreground and background colors */
 		cfg->color_set = 0;
 	else {
-		if (!gdk_color_parse (fore_color, &cfg->user_fore) || !gdk_color_parse (back_color, &cfg->user_back)){
+		if (!gdk_color_parse (fore_color, &cfg->palette[16])
+		    || !gdk_color_parse (back_color, &cfg->palette[17])){
 			/* or illegal colors */
 			cfg->color_set = 0;
 		}
 	}
 
+	/* load the palette, if none, then 'default' to safe (linux) pallete */
+	gnome_config_get_vector("palette", &colour_count, &colours);
+	for (i=0;i<18;i++) {
+		if (i<colour_count)
+			gdk_color_parse(colours[i], &cfg->palette[i]);
+		else if (i<16) {
+			cfg->palette[i].red = linux_red[i];
+			cfg->palette[i].green = linux_grn[i];
+			cfg->palette[i].blue = linux_blu[i];
+		}
+	}
+	g_free(colours);
+
 	return cfg;
 }
 
@@ -477,6 +528,7 @@ gather_changes (ZvtTerm *term)
 	preferences_t *prefs = gtk_object_get_data (GTK_OBJECT (term), "prefs");
 	gushort r, g, b;
 	struct terminal_config *newcfg = g_malloc (sizeof (*newcfg));
+	int i;
 
 	memset (newcfg, 0, sizeof (*newcfg));
 
@@ -513,14 +565,13 @@ gather_changes (ZvtTerm *term)
 	newcfg->color_type = option_menu_get_history (GTK_OPTION_MENU (prefs->color_scheme));
 	newcfg->color_set  = option_menu_get_history (GTK_OPTION_MENU (prefs->def_fore_back));
 
-	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (prefs->fore_cs), &r, &g, &b, NULL);
-	newcfg->user_fore.red   = r;
-	newcfg->user_fore.green = g;
-	newcfg->user_fore.blue  = b;
-	gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (prefs->back_cs), &r, &g, &b, NULL);
-	newcfg->user_back.red   = r;
-	newcfg->user_back.green = g;
-	newcfg->user_back.blue  = b;
+	for (i=0;i<18;i++) {
+		gnome_color_picker_get_i16 (GNOME_COLOR_PICKER (prefs->palette[i]), &r, &g,
+					    &b, NULL);
+		newcfg->palette[i].red   = r;
+		newcfg->palette[i].green = g;
+		newcfg->palette[i].blue  = b;
+	}
 
 	return newcfg;
 }
@@ -566,7 +617,7 @@ apply_changes (ZvtTerm *term, struct terminal_config *newcfg)
 	gtk_object_set_data (GTK_OBJECT (term), "config", cfg);
 
 	gnome_term_set_font (term, cfg->font);
-	zvt_term_set_wordclass (term, cfg->wordclass);
+	zvt_term_set_wordclass (term, (guchar *)cfg->wordclass);
 	zvt_term_set_bell(term, !cfg->bell);
 	zvt_term_set_blink (term, cfg->blink);
 	zvt_term_set_scroll_on_keystroke (term, cfg->scroll_key);
@@ -790,13 +841,18 @@ typedef struct {
 static void
 check_color_sensitivity (preferences_t *prefs)
 {
-	int idx = option_menu_get_history (GTK_OPTION_MENU (prefs->def_fore_back));
-	gboolean sens = (idx == COLORS_CUSTOM);
+	int idxc = option_menu_get_history (GTK_OPTION_MENU (prefs->def_fore_back));
+	gboolean sensc = (idxc == COLORS_CUSTOM);
+	int idx = option_menu_get_history (GTK_OPTION_MENU (prefs->color_scheme));
+	gboolean sens = (idx == PALETTE_CUSTOM);
 
-	gtk_widget_set_sensitive (GTK_WIDGET (prefs->fore_label), sens);
-	gtk_widget_set_sensitive (GTK_WIDGET (prefs->fore_cs), sens);
-	gtk_widget_set_sensitive (GTK_WIDGET (prefs->back_label), sens);
-	gtk_widget_set_sensitive (GTK_WIDGET (prefs->back_cs), sens);
+	int i;
+
+	gtk_widget_set_sensitive (GTK_WIDGET (prefs->fore_label), sensc);
+	gtk_widget_set_sensitive (GTK_WIDGET (prefs->back_label), sensc);
+	gtk_widget_set_sensitive (GTK_WIDGET (prefs->palette_label), sens);
+	for (i=0;i<18;i++)
+		gtk_widget_set_sensitive (GTK_WIDGET (prefs->palette[i]), i<16?sens:sensc);
 }
 
 static void
@@ -854,6 +910,7 @@ char *color_scheme [] = {
 	N_("Linux console"),
 	N_("Color Xterm"),
 	N_("rxvt"),
+	N_("Custom"),
 	NULL
 };
 
@@ -901,19 +958,14 @@ free_cs (GtkWidget *widget, GnomeColorPicker *cp)
 {
 }
 	 
-static char *
-get_color_string (GdkColor c)
-{
-	static char buffer [40];
-
-	sprintf (buffer, "rgb:%04x/%04x/%04x", c.red, c.green , c.blue);
-	return buffer;
-}
-
 static void
 save_preferences (GtkWidget *widget, ZvtTerm *term, 
 		  struct terminal_config *cfg)
 {
+	char *colours[18];
+	int i;
+	char *scheme[4] = { "linux", "xterm", "rxvt", "custom" };
+
 	if (cfg->font)
 		gnome_config_set_string ("font", cfg->font);
 	if (cfg->wordclass)
@@ -927,13 +979,9 @@ save_preferences (GtkWidget *widget, ZvtTerm *term,
 	gnome_config_set_bool   ("login_by_default", cfg->login_by_default);
 	gnome_config_set_int    ("scrollbacklines", cfg->scrollback);
 	gnome_config_set_int    ("color_set", cfg->color_set);
-	gnome_config_set_string ("color_scheme",
-				 cfg->color_type == 0 ? "linux" : 
-				 (cfg->color_type == 1 ? "xterm" : "rxvt"));
-	gnome_config_set_string ("foreground", 
-				 get_color_string (cfg->user_fore));
-	gnome_config_set_string ("background", 
-				 get_color_string (cfg->user_back));
+	if (cfg->color_type>=4)
+		cfg->color_type=4;
+	gnome_config_set_string ("color_scheme", scheme[cfg->color_type]);
 	gnome_config_set_bool   ("menubar", !cfg->menubar_hidden);
 	gnome_config_set_bool   ("scrollonkey", cfg->scroll_key);
 	gnome_config_set_bool   ("scrollonoutput", cfg->scroll_out);
@@ -944,6 +992,14 @@ save_preferences (GtkWidget *widget, ZvtTerm *term,
 #endif
 	gnome_config_set_bool   ("background_pixmap", cfg->background_pixmap);
 	gnome_config_set_string ("pixmap_file", cfg->pixmap_file);
+
+	for (i=0;i<18;i++)
+		colours[i] = g_strdup_printf ("rgb:%04x/%04x/%04x", cfg->palette[i].red,
+					      cfg->palette[i].green, cfg->palette[i].blue);
+	gnome_config_set_vector ("palette", 18, (const char **)colours);
+	for (i=0;i<18;i++)
+		g_free(colours[i]);
+
 	gnome_config_sync ();
 }
 
@@ -964,13 +1020,14 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 {
 	static GnomeHelpMenuEntry help_entry = { NULL, "properties" };
 	GtkWidget *l, *table, *picker, *label, *b1, *b2;
-	GtkWidget *r, *frame, *hbox, *subtable;
+	GtkWidget *r, *frame, *hbox, *subtable, *paltable;
 	preferences_t *prefs;
 	GtkAdjustment *adj;
 	GList *class_list = NULL;
 	void *iter;
 	char *some_class;
 	struct terminal_config *cfg;
+	int i;
 
 	/* Is a property window for this terminal already running? */
 	if (gtk_object_get_data (GTK_OBJECT (term), "newcfg"))
@@ -1006,8 +1063,10 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	gtk_entry_set_position (GTK_ENTRY (prefs->font_entry), 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->font_entry), "changed",
 			    GTK_SIGNAL_FUNC (font_changed), prefs);
-		gtk_table_attach (GTK_TABLE (table), prefs->font_entry,
+	gtk_table_attach (GTK_TABLE (table), prefs->font_entry,
 			  2, 3, FONT_ROW, FONT_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
+	gnome_dialog_editable_enters (GNOME_DIALOG (prefs->prop_win),
+				      GTK_EDITABLE (prefs->font_entry));
 
 	picker = gnome_font_picker_new();
 	gnome_font_picker_set_font_name(GNOME_FONT_PICKER(picker),
@@ -1032,12 +1091,12 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	iter = gnome_config_init_iterator_sections ("/Terminal");
 	while (gnome_config_iterator_next (iter, &some_class, NULL)){
 		if (!strcmp (some_class, "Config") || !strncmp (some_class, "Class-", 6)){
-		    if (!strcmp (some_class, "Config")) 
-			    some_class = _("Default");
-		    else
-			    some_class += 6;
+			if (!strcmp (some_class, "Config")) 
+				some_class = _("Default");
+			else
+				some_class += 6;
 
-		    class_list = g_list_append (class_list, some_class);
+			class_list = g_list_append (class_list, some_class);
 		}
 	}
 	if(class_list)
@@ -1055,7 +1114,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	/* Blinking status */
 	prefs->blink_checkbox = gtk_check_button_new_with_label (_("Blinking cursor"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->blink_checkbox),
-				     term->blink_enabled ? 1 : 0);
+				      term->blink_enabled ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->blink_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->blink_checkbox,
@@ -1064,7 +1123,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	/* Show menu bar */
 	prefs->menubar_checkbox = gtk_check_button_new_with_label (_("Hide menu bar"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->menubar_checkbox),
-				     cfg->menubar_hidden ? 1 : 0);
+				      cfg->menubar_hidden ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->menubar_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->menubar_checkbox,
@@ -1073,7 +1132,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	/* Toggle the bell */
 	prefs->bell_checkbox = gtk_check_button_new_with_label (_("Silence Terminal bell"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->bell_checkbox),
-				     zvt_term_get_bell(term) ? 0 : 1);
+				      zvt_term_get_bell(term) ? 0 : 1);
 	gtk_signal_connect (GTK_OBJECT (prefs->bell_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->bell_checkbox,
@@ -1082,7 +1141,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	/* Swap keys */
 	prefs->swapkeys_checkbox = gtk_check_button_new_with_label (_("Swap DEL/Backspace"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->swapkeys_checkbox),
-				     cfg->swap_keys ? 1 : 0);
+				      cfg->swap_keys ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->swapkeys_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->swapkeys_checkbox,
@@ -1108,7 +1167,10 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->wordclass_entry,
 			  2, 3, WORDCLASS_ROW, WORDCLASS_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
-	
+	gnome_dialog_editable_enters (GNOME_DIALOG (prefs->prop_win),
+				      GTK_EDITABLE (prefs->wordclass_entry));
+
+
 	/* Image page */
 	/* if pixmap support isn't in zvt, we still create the widgets for
 	   the page, so that we can query them later, but they won't be shown
@@ -1138,7 +1200,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 		gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (r),
 							     _("Background pixmap"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->pixmap_checkbox),
-				     term->pixmap_filename ? 1 : 0);
+				      term->pixmap_filename ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->pixmap_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (subtable), r,
@@ -1169,7 +1231,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 #ifdef ZVT_BACKGROUND_SCROLL
 	prefs->pixmap_scrollable_checkbox = gtk_check_button_new_with_label (_("Background pixmap scrolls"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->pixmap_scrollable_checkbox),
-				     cfg->scroll_background ? 1 : 0);
+				      cfg->scroll_background ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->pixmap_scrollable_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (subtable), prefs->pixmap_scrollable_checkbox,
@@ -1182,7 +1244,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 		gtk_radio_button_new_with_label_from_widget (GTK_RADIO_BUTTON (r),
 							     _("Transparent"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->transparent_checkbox),
-				     term->transparent ? 1 : 0);
+				      term->transparent ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->transparent_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (subtable), r,
@@ -1209,7 +1271,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	/* Shaded */
 	prefs->shaded_checkbox = gtk_check_button_new_with_label (_("Background should be shaded"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->shaded_checkbox),
-				     term->shaded ? 1 : 0);
+				      term->shaded ? 1 : 0);
 	gtk_signal_connect (GTK_OBJECT (prefs->shaded_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->shaded_checkbox,
@@ -1223,7 +1285,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	gnome_property_box_append_page (GNOME_PROPERTY_BOX (prefs->prop_win), table, gtk_label_new (_("Colors")));
 	
 	/* Color palette */
-	l = aligned_label (_("Color palette:"));
+	l = aligned_label (_("Color scheme:"));
 	gtk_table_attach (GTK_TABLE (table), l,
 			  1, 2, COLORPAL_ROW, COLORPAL_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
 	prefs->color_scheme = create_option_menu (GNOME_PROPERTY_BOX (prefs->prop_win), prefs,
@@ -1236,25 +1298,41 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	prefs->fore_label = aligned_label (_("Foreground color:"));
 	gtk_table_attach (GTK_TABLE (table), prefs->fore_label,
 			  1, 2, FORECOLOR_ROW, FORECOLOR_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
-	prefs->fore_cs = gnome_color_picker_new ();
-	gtk_table_attach (GTK_TABLE (table), b1 = prefs->fore_cs,
+	prefs->palette[16] = gnome_color_picker_new ();
+	gtk_table_attach (GTK_TABLE (table), b1 = prefs->palette[16],
 			  2, 3, FORECOLOR_ROW, FORECOLOR_ROW+1, 0, 0, GNOME_PAD, GNOME_PAD);
-	gtk_signal_connect (GTK_OBJECT (b1), "destroy", GTK_SIGNAL_FUNC (free_cs), prefs->fore_cs);
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs->fore_cs), cfg->user_fore.red, cfg->user_fore.green, cfg->user_fore.blue, 0);
-	
+	gtk_signal_connect (GTK_OBJECT (b1), "destroy", GTK_SIGNAL_FUNC (free_cs), prefs->palette[16]);
+
 	prefs->back_label = aligned_label (_("Background color:"));
 	gtk_table_attach (GTK_TABLE (table), prefs->back_label,
 			  1, 2, BACKCOLOR_ROW, BACKCOLOR_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
-	prefs->back_cs = gnome_color_picker_new ();
-	gtk_table_attach (GTK_TABLE (table), b2 = prefs->back_cs,
+	prefs->palette[17] = gnome_color_picker_new ();
+	gtk_table_attach (GTK_TABLE (table), b2 = prefs->palette[17],
 			  2, 3, BACKCOLOR_ROW, BACKCOLOR_ROW+1, 0, 0, GNOME_PAD, GNOME_PAD);
-	gtk_signal_connect (GTK_OBJECT (b2), "destroy", GTK_SIGNAL_FUNC (free_cs), prefs->back_cs);
-	gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs->back_cs), cfg->user_back.red, cfg->user_back.green, cfg->user_back.blue, 0);
-	gtk_signal_connect (GTK_OBJECT (prefs->fore_cs), "color_set", GTK_SIGNAL_FUNC (color_changed), prefs);
-	gtk_signal_connect (GTK_OBJECT (prefs->back_cs), "color_set", GTK_SIGNAL_FUNC (color_changed), prefs);
+	gtk_signal_connect (GTK_OBJECT (b2), "destroy", GTK_SIGNAL_FUNC (free_cs), prefs->palette[17]);
+	l = aligned_label (_("Color palette:"));
+	gtk_table_attach (GTK_TABLE (table), l,
+			  1, 2, BACKCOLOR_ROW+1, BACKCOLOR_ROW+2, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
+	prefs->palette_label = l;
+
+	paltable = gtk_table_new(8,2,FALSE);
+	gtk_table_attach (GTK_TABLE (table), paltable,
+			  2, 3, BACKCOLOR_ROW+1, BACKCOLOR_ROW+2, GTK_FILL, 0,
+			  GNOME_PAD, GNOME_PAD);
+	for (i=0;i<18;i++) {
+		if (i<16) {
+			prefs->palette[i] = gnome_color_picker_new();
+			gtk_table_attach(GTK_TABLE(paltable), prefs->palette[i], i&7, (i&7)+1,
+					 i/8, i/8+1, 0, 0, 0, 0);
+		}
+		gnome_color_picker_set_i16(GNOME_COLOR_PICKER(prefs->palette[i]), cfg->palette[i].red,
+					   cfg->palette[i].green, cfg->palette[i].blue, 0);
+		gtk_signal_connect (GTK_OBJECT (prefs->palette[i]), "color_set",
+				    GTK_SIGNAL_FUNC (color_changed), prefs);
+	}
 
 	/* default foreground/backgorund selector */
-	l = aligned_label (_("Colors:"));
+	l = aligned_label (_("Fore/Background Colour:"));
 	gtk_table_attach (GTK_TABLE (table), l,
 			  1, 2, FOREBACK_ROW, FOREBACK_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
 	prefs->def_fore_back = create_option_menu_data (GNOME_PROPERTY_BOX (prefs->prop_win), prefs,
@@ -1291,11 +1369,13 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->scrollback_spin, 2, 3, SCROLLBACK_ROW, 
 			  SCROLLBACK_ROW+1, GTK_FILL, 0, GNOME_PAD, GNOME_PAD);
+	gnome_dialog_editable_enters (GTK_DIALOG (prefs->prop_win),
+				      GTK_EDITABLE (prefs->scrollback_spin));
 
 	/* Scroll on keystroke checkbox */
 	prefs->scroll_kbd_checkbox = gtk_check_button_new_with_label (_("Scroll on keystroke"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->scroll_kbd_checkbox),
-				     cfg->scroll_key);
+				      cfg->scroll_key);
 	gtk_signal_connect (GTK_OBJECT (prefs->scroll_kbd_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->scroll_kbd_checkbox, 2, 3, 
@@ -1304,7 +1384,7 @@ preferences_cmd (GtkWidget *widget, ZvtTerm *term)
 	/* Scroll on output checkbox */
 	prefs->scroll_out_checkbox = gtk_check_button_new_with_label (_("Scroll on output"));
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs->scroll_out_checkbox),
-				     cfg->scroll_out);
+				      cfg->scroll_out);
 	gtk_signal_connect (GTK_OBJECT (prefs->scroll_out_checkbox), "toggled",
 			    GTK_SIGNAL_FUNC (prop_changed), prefs);
 	gtk_table_attach (GTK_TABLE (table), prefs->scroll_out_checkbox, 2, 3, 
@@ -1409,6 +1489,20 @@ toggle_secure_keyboard_cmd (GtkWidget *w, ZvtTerm *term)
 		gdk_keyboard_ungrab (GDK_CURRENT_TIME);
 }
 
+#ifdef ZVT_TERM_MATCH_SUPPORT
+static void
+load_url_cmd (GtkWidget *widget, ZvtTerm *term)
+{
+	char *url;
+
+	url = gtk_object_get_data (GTK_OBJECT (term), "matchstr");
+	if (url) {
+		gnome_url_show(url);
+	}
+
+}
+#endif
+
 static GnomeUIInfo gnome_terminal_terminal_menu [] = {
         GNOMEUIINFO_MENU_NEW_ITEM (N_("_New terminal"), N_("Creates a new terminal window"), new_terminal, NULL),
 	GNOMEUIINFO_SEPARATOR,
@@ -1437,10 +1531,19 @@ static GnomeUIInfo gnome_terminal_popup_menu [] = {
 #ifdef HAVE_ZVT_TERM_RESET
 	GNOMEUIINFO_ITEM_NONE (N_("_Reset Terminal"), NULL, reset_terminal_soft_cmd),
 	GNOMEUIINFO_ITEM_NONE (N_("Reset and _Clear"), NULL, reset_terminal_hard_cmd),
+#endif
+#ifdef ZVT_TERM_MATCH_SUPPORT
+	GNOMEUIINFO_END,	/* used as a free slot for dymanic menu item */
 #endif
 	GNOMEUIINFO_END
 };
 
+#ifdef ZVT_TERM_MATCH_SUPPORT
+static GnomeUIInfo gnome_terminal_popup_menu_url [] = {
+	GNOMEUIINFO_ITEM_NONE (N_("_Open in browser"), NULL, load_url_cmd),
+};
+#endif
+
 static GnomeUIInfo gnome_terminal_help_menu [] = {
 	GNOMEUIINFO_HELP ("gnome-terminal"),
 	GNOMEUIINFO_MENU_ABOUT_ITEM(about_terminal_cmd, NULL),
@@ -1462,15 +1565,30 @@ static GnomeUIInfo gnome_terminal_settings_menu [] = {
 	GNOMEUIINFO_END
 };
 
-static GnomeUIInfo gnome_terminal_menu [] = {
-	GNOMEUIINFO_MENU_FILE_TREE(gnome_terminal_terminal_menu),
-	GNOMEUIINFO_MENU_EDIT_TREE(gnome_terminal_edit),
-	GNOMEUIINFO_MENU_SETTINGS_TREE(gnome_terminal_settings_menu),
-	GNOMEUIINFO_MENU_HELP_TREE(gnome_terminal_help_menu),
-	GNOMEUIINFO_END
+static GnomeUIInfo gnome_terminal_menu[] = {
+   {
+	GNOME_APP_UI_SUBTREE_STOCK, N_("File"), NULL,
+	gnome_terminal_terminal_menu, NULL, NULL, (GnomeUIPixmapType) 0,
+	NULL, 0, (GdkModifierType) 0, NULL
+   },
+   {
+	GNOME_APP_UI_SUBTREE_STOCK, N_("Edit"), NULL,
+	gnome_terminal_edit, NULL, NULL, (GnomeUIPixmapType) 0, NULL, 0,
+	(GdkModifierType) 0, NULL
+   },
+   {
+	GNOME_APP_UI_SUBTREE_STOCK, N_("Settings"), NULL,
+        gnome_terminal_settings_menu, NULL, NULL,
+	(GnomeUIPixmapType) 0, NULL, 0, (GdkModifierType) 0, NULL
+   },
+   {
+	GNOME_APP_UI_SUBTREE_STOCK, N_("Help"), NULL,
+	gnome_terminal_help_menu, NULL, NULL, (GnomeUIPixmapType) 0,
+	NULL, 0, (GdkModifierType) 0, NULL
+   },
+   GNOMEUIINFO_END
 };
 
-
 /*
  * Puts in *shell a pointer to the full shell pathname
  * Puts in *name the invocation name for the shell
@@ -1565,6 +1683,7 @@ drag_data_received  (GtkWidget *widget, GdkDragContext *context,
 	{
 		guint16 *data = (guint16 *)selection_data->data;
 		preferences_t *prefs;
+		int i;
 
 		if (selection_data->length != 8)
 			return;
@@ -1576,40 +1695,25 @@ drag_data_received  (GtkWidget *widget, GdkDragContext *context,
 		cfg->color_set = COLORS_CUSTOM;
 		
 		the_char = vt_get_attr_at (term->vx, col, row) & 0xff;
-		if (the_char == ' ' || the_char == 0){
-			/* copy the current foreground color */
-			cfg->user_fore.red   = red [16];
-			cfg->user_fore.green = green [16];
-			cfg->user_fore.blue  = blue [16];
+		if (the_char == ' ' || the_char == 0)
+			i=17;
+		else
+			i=16;
+
+		cfg->palette[i].red = data[0];
+		cfg->palette[i].green = data[1];
+		cfg->palette[i].blue = data[2];
 
-			/* Accept the dropped colors */
-			cfg->user_back.red   = data [0];
-			cfg->user_back.green = data [1];
-			cfg->user_back.blue  = data [2];
-		} else {
-			/* copy the current background color */
-			cfg->user_back.red   = red [17];
-			cfg->user_back.green = green [17];
-			cfg->user_back.blue  = blue [17];
-			
-			/* Accept the dropped colors */
-			cfg->user_fore.red   = data [0];
-			cfg->user_fore.green = data [1];
-			cfg->user_fore.blue  = data [2];
-		}
 		set_color_scheme (term, cfg);
 		save_preferences_cmd (widget, term);
 
 		prefs  = gtk_object_get_data (GTK_OBJECT (term), "prefs");
 		if (prefs) {
-			gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs->fore_cs),
-						     cfg->user_fore.red,
-						     cfg->user_fore.green,
-						     cfg->user_fore.blue, 0);
-			gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs->back_cs),
-						     cfg->user_back.red,
-						     cfg->user_back.green,
-						     cfg->user_back.blue, 0);
+			for (i=16;i<18;i++)
+				gnome_color_picker_set_i16 (GNOME_COLOR_PICKER (prefs->palette[i]),
+							    cfg->palette[i].red,
+							    cfg->palette[i].green,
+							    cfg->palette[i].blue, 0);
 			gtk_option_menu_set_history (GTK_OPTION_MENU (prefs->def_fore_back),
 						     cfg->color_set);
 			check_color_sensitivity (prefs);
@@ -1642,9 +1746,27 @@ static int
 button_press (GtkWidget *widget, GdkEventButton *event, ZvtTerm *term)
 {
 	GtkWidget *menu;
-	struct terminal_config *cfg;
 	GnomeUIInfo *uiinfo;
+	struct terminal_config *cfg;
 	GtkCheckMenuItem *toggle_item;
+#ifdef ZVT_TERM_MATCH_SUPPORT
+	char *match;
+	int x,y;
+	GdkModifierType mask;
+#endif
+
+#ifdef ZVT_TERM_MATCH_SUPPORT
+	gdk_window_get_pointer(widget->window, &x, &y, &mask);
+	match = zvt_term_match_check(term, x/term->charwidth, y/term->charheight, 0);
+
+	if (event->button == 1
+	    && (event->state & GDK_CONTROL_MASK)
+	    && match) {
+		gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
+		gnome_url_show (match);
+		return TRUE;
+	}
+#endif /* ZVT_TERM_MATCH_SUPPORT */
 
 	if (event->button != 3
 	    || (!(event->state & GDK_CONTROL_MASK) && term->vx->selected)
@@ -1653,12 +1775,25 @@ button_press (GtkWidget *widget, GdkEventButton *event, ZvtTerm *term)
 
 	gtk_signal_emit_stop_by_name (GTK_OBJECT (widget), "button_press_event");
 
-	cfg = gtk_object_get_data (GTK_OBJECT (term), "config");
-
-
+#ifdef ZVT_TERM_MATCH_SUPPORT
+	/* construct the popup menu properly */
+	if (match) {
+		memcpy(&gnome_terminal_popup_menu[POPUP_MENU_DYNAMIC_INDEX],
+		       &gnome_terminal_popup_menu_url[0],
+		       sizeof(gnome_terminal_popup_menu_url));
+		
+		gtk_object_set_data (GTK_OBJECT (term), "matchstr", match);
+	} else {
+		/* make sure the optional menu isn't there */
+		memcpy(&gnome_terminal_popup_menu[POPUP_MENU_DYNAMIC_INDEX],
+		       &gnome_terminal_popup_menu[POPUP_MENU_LAST_INDEX],
+		       sizeof(gnome_terminal_popup_menu[0]));
+	}
+#endif
 	uiinfo = gnome_terminal_popup_menu;
+	menu = gnome_popup_menu_new (gnome_terminal_popup_menu);
 
-	menu = gnome_popup_menu_new (uiinfo);
+	cfg = gtk_object_get_data (GTK_OBJECT (term), "config");
 
 	/*
 	 * Set the toggle state for the "show menubar"
@@ -1677,6 +1812,13 @@ button_press (GtkWidget *widget, GdkEventButton *event, ZvtTerm *term)
 	gnome_popup_menu_do_popup_modal (menu, NULL, NULL, event, term);
 	gtk_widget_destroy (menu);
 
+	/*
+	 * The popup menu ungrabs the keyboard if it was grabbed,
+	 * so we grab it again here.
+	 */
+	if (cfg->keyboard_secured)
+		gdk_keyboard_grab (term->term_window, TRUE, GDK_CURRENT_TIME);
+
 	return TRUE;
 }
 
@@ -1871,7 +2013,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 
 	zvt_term_set_scrollback (term, cfg->scrollback);
 	gnome_term_set_font (term, cfg->font);
-	zvt_term_set_wordclass  (term, cfg->wordclass);
+	zvt_term_set_wordclass  (term, (guchar *)cfg->wordclass);
 	zvt_term_set_bell  (term, !cfg->bell);
 	zvt_term_set_blink (term, cfg->blink);
 	zvt_term_set_scroll_on_keystroke (term, cfg->scroll_key);
@@ -1893,6 +2035,11 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	gtk_signal_connect_after (GTK_OBJECT (term), "realize",
 				  GTK_SIGNAL_FUNC (set_hints), term);
 
+#ifdef ZVT_TERM_MATCH_SUPPORT
+	zvt_term_match_add( ZVT_TERM(term), "(((news|telnet|nttp|file|http|ftp)://)|(www|ftp))[-A-Za-z0-9\\.]+(:[0-9]*)?", VTATTR_UNDERLINE, "host only url");
+	zvt_term_match_add( ZVT_TERM(term), "(((news|telnet|nttp|file|http|ftp)://)|(www|ftp))[-A-Za-z0-9\\.]+(:[0-9]*)?/[-A-Za-z0-9_\\$\\.\\+\\!\\*\\(\\),;:@&=\\?/~\\#]*[^]'\\.}>\\) ,\\\"]", VTATTR_UNDERLINE, "full url");
+#endif
+
 	if (!cfg->menubar_hidden)
 		gnome_app_create_menus_with_data (GNOME_APP (app), gnome_terminal_menu, term);
 	
@@ -1956,7 +2103,7 @@ new_terminal_cmd (char **cmd, struct terminal_config *cfg_in, gchar *geometry)
 	XSync(GDK_DISPLAY(), False);
 
 	errno = 0;
-	switch (zvt_term_forkpty (term, update_records)){
+	switch (zvt_term_forkpty (term, cfg->update_records)){
 	case -1:
 		show_pty_error_dialog(errno);
 		/* should we exit maybe? */
@@ -2035,6 +2182,11 @@ load_session ()
 		/* do this now, since it is only done in a session */
 		cfg->invoke_as_login_shell = gnome_config_get_bool("invoke_as_login_shell");
 		cfg->termname = gnome_config_get_string("termname");
+		cfg->window_title = gnome_config_get_string("window_title");
+		if (gnome_config_get_bool ("do_utmp=true"))
+			cfg->update_records |= ZVT_TERM_DO_UTMP_LOG;
+		if (gnome_config_get_bool ("do_wtmp=true"))
+			cfg->update_records |= ZVT_TERM_DO_WTMP_LOG;
 
 		g_free(class);
 		gnome_config_pop_prefix();
@@ -2100,7 +2252,7 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
 		  term->charheight;
 
 		geom = g_string_new ("");
-		g_string_sprintf (geom, "%dx%d+%d+%d", width, height, x, y);
+		g_string_sprintf (geom, "%dx%d%+d%+d", width, height, x, y);
 		gnome_config_set_string ("geometry", geom->str);
 		g_string_free (geom, TRUE);
 		
@@ -2116,10 +2268,11 @@ save_session (GnomeClient *client, gint phase, GnomeSaveStyle save_style,
 		save_preferences(list->data, term, cfg);
 		
 		/* do this now, since it is only done in a session */
-		gnome_config_set_bool("invoke_as_login_shell",
-				      cfg->invoke_as_login_shell);
-		gnome_config_set_string("termname",
-					cfg->termname);
+		gnome_config_set_bool("invoke_as_login_shell", cfg->invoke_as_login_shell);
+		gnome_config_set_string("termname", cfg->termname?cfg->termname:"xterm");
+		gnome_config_set_string("window_title", cfg->window_title?cfg->window_title:"Terminal");
+		gnome_config_set_bool("do_utmp", (cfg->update_records & ZVT_TERM_DO_UTMP_LOG) != 0);
+		gnome_config_set_bool("do_wtmp", (cfg->update_records & ZVT_TERM_DO_WTMP_LOG) != 0);
 
 		gnome_config_pop_prefix ();
 		g_free (prefix);
@@ -2173,7 +2326,7 @@ enum {
 };
 
 static struct poptOption cb_options [] = {
-	{ NULL, '\0', POPT_ARG_CALLBACK, parse_an_arg, 0},
+	{ NULL, '\0', POPT_ARG_CALLBACK, (gpointer)parse_an_arg, 0},
 
 	{ "tclass", '\0', POPT_ARG_STRING, NULL, CLASS_KEY,
 	  N_("Terminal class name"), N_("TCLASS")},
@@ -2182,10 +2335,10 @@ static struct poptOption cb_options [] = {
 	  N_("Specifies font name"), N_("FONT")},
 
 	{ "nologin", '\0', POPT_ARG_NONE, NULL, NOLOGIN_KEY,
-	  N_("Do not start up shells as logins shells"), NULL},
+	  N_("Do not start up shells as login shells"), NULL},
 
 	{ "login", '\0', POPT_ARG_NONE, NULL, LOGIN_KEY,
-	  N_("Start up shells as logins shells"), NULL},
+	  N_("Start up shells as login shells"), NULL},
 
 	{ "geometry", '\0', POPT_ARG_STRING, NULL, GEOMETRY_KEY,
 	  N_("Specifies the geometry for the main window"), N_("GEOMETRY")},
@@ -2194,7 +2347,7 @@ static struct poptOption cb_options [] = {
 	  N_("Execute this program instead of a shell"), N_("COMMAND")},
 
 	{ "execute", 'x', POPT_ARG_STRING, NULL, COMMAND_KEY,
-	  N_("Execute this program the same was as xterm does"), N_("COMMAND")},
+	  N_("Execute this program the same way as xterm does"), N_("COMMAND")},
 
 	{ "foreground", '\0', POPT_ARG_STRING, NULL, FORE_KEY,
 	  N_("Foreground color"), N_("COLOR")},
@@ -2267,26 +2420,24 @@ parse_an_arg (poptContext state,
 		break;
 		  }
 	case FORE_KEY:
-	        cfg->user_fore_str = arg;
-		/* gdk_color_parse(cfg->user_fore_str, &cfg->user_fore); */
+		cfg->user_fore_str = arg;
 		cfg->have_user_colors = 1;
 		break;
 	case BACK_KEY:
-	        cfg->user_back_str = arg;
-		/* gdk_color_parse(cfg->user_back_str, &cfg->user_back); */
+		cfg->user_back_str = arg;
 		cfg->have_user_colors = 1;
 		break;
 	case DOUTMP_KEY:
-		update_records |= ZVT_TERM_DO_UTMP_LOG;
+		cfg->update_records |= ZVT_TERM_DO_UTMP_LOG;
 		break;
 	case DONOUTMP_KEY:
-		update_records &= ~ZVT_TERM_DO_UTMP_LOG;
+		cfg->update_records &= ~ZVT_TERM_DO_UTMP_LOG;
 		break;
 	case DOWTMP_KEY:
-		update_records |= ZVT_TERM_DO_WTMP_LOG;
+		cfg->update_records |= ZVT_TERM_DO_WTMP_LOG;
 		break;
 	case DONOWTMP_KEY:
-		update_records &= ~ZVT_TERM_DO_WTMP_LOG;
+		cfg->update_records &= ~ZVT_TERM_DO_WTMP_LOG;
 		break;
 	case TITLE_KEY:
 	        cfg->window_title = g_strdup(arg);
@@ -2295,6 +2446,7 @@ parse_an_arg (poptContext state,
 		cfg->termname = g_strdup_printf("TERM=%s", arg);
 		break;
 	default:
+		break;
 	}
 }
 
@@ -2350,15 +2502,14 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	gnome_init_with_popt_table("Terminal", VERSION, argc, argv,
 				   cb_options, 0, NULL);
 
-#if 1
 	if(cmdline_config->user_back_str)
 		gdk_color_parse(cmdline_config->user_back_str,
-				&cmdline_config->user_back);
+				&cmdline_config->palette[17]);
 	
 	if(cmdline_config->user_fore_str)
 		gdk_color_parse(cmdline_config->user_fore_str,
-				&cmdline_config->user_fore);
-#endif
+				&cmdline_config->palette[16]);
+
 	if (cmdline_config->class){
 		class = g_strdup (cmdline_config->class);
 	}
@@ -2410,8 +2561,8 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	
 	if (cmdline_config->have_user_colors){
 		default_config->color_set = cmdline_config->color_set;
-		default_config->user_fore = cmdline_config->user_fore;
-		default_config->user_back = cmdline_config->user_back;
+		default_config->palette[16] = cmdline_config->palette[16];
+		default_config->palette[17] = cmdline_config->palette[17];
 		default_config->color_set = COLORS_CUSTOM;
 	}
 
@@ -2422,6 +2573,8 @@ main_terminal_program (int argc, char *argv [], char **environ)
 		default_config->login_by_default;
 	default_config->termname = g_strdup(cmdline_config->termname);
 
+	default_config->update_records = cmdline_config->update_records;
+
 	terminal_config_free (cmdline_config);
 	
 	if (!load_session ())
@@ -2433,8 +2586,10 @@ main_terminal_program (int argc, char *argv [], char **environ)
 	return 0;
 }
 
+extern char **environ;
+
 int
-main (int argc, char *argv [], char **environ)
+main (int argc, char *argv [])
 {
 	return main_terminal_program (argc, argv, environ);
 }
