@@ -81,12 +81,12 @@ server_applet__get_goad_id(CustomAppletServant *servant,
 
 static POA_GNOME_Applet__epv applet_epv = {
   NULL,
-  (gpointer)&server_applet_change_orient,
-  (gpointer)&server_applet_do_callback,
-  (gpointer)&server_applet_session_save,
-  (gpointer)&server_applet_back_change,
-  (gpointer)&server_applet_set_tooltips_state,
-  (gpointer)&server_applet__get_goad_id
+  &server_applet_change_orient,
+  &server_applet_do_callback,
+  &server_applet_session_save,
+  &server_applet_back_change,
+  &server_applet_set_tooltips_state,
+  &server_applet__get_goad_id
 };
 
 static POA_GNOME_Applet__vepv vepv = { &base_epv, &applet_epv };
@@ -139,8 +139,9 @@ applet_widget_get_type ()
 			sizeof (AppletWidgetClass),
 			(GtkClassInitFunc) applet_widget_class_init,
 			(GtkObjectInitFunc) wapplet_widget_init,
-			(GtkArgSetFunc) NULL,
-			(GtkArgGetFunc) NULL,
+			NULL,
+			NULL,
+			(GtkClassInitFunc) NULL
 		};
 
 		applet_widget_type = gtk_type_unique (gtk_plug_get_type (), &applet_widget_info);
@@ -157,7 +158,7 @@ enum {
 	LAST_SIGNAL
 };
 
-static int applet_widget_signals[LAST_SIGNAL] = {0,0,0,0};
+static guint applet_widget_signals[LAST_SIGNAL] = {0,0,0,0};
 
 static void
 applet_widget_marshal_signal_orient (GtkObject * object,
@@ -575,6 +576,8 @@ applet_widget_register_callback_dir(AppletWidget *applet,
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
+	g_return_if_fail(name != NULL);
+	g_return_if_fail(menutext != NULL);
 
 	gnome_panel_applet_register_callback_dir (GTK_WIDGET(applet),name,
 						  "",menutext);
@@ -587,6 +590,9 @@ applet_widget_register_stock_callback_dir(AppletWidget *applet,
 {
 	g_return_if_fail(applet != NULL);
 	g_return_if_fail(IS_APPLET_WIDGET(applet));
+	g_return_if_fail(name != NULL);
+	g_return_if_fail(stock_type != NULL);
+	g_return_if_fail(menutext != NULL);
 
 	gnome_panel_applet_register_callback_dir (GTK_WIDGET(applet),name,
 						  stock_type,menutext);
@@ -662,10 +668,6 @@ gnome_panel_applet_corba_init(AppletWidget *applet, const char *goad_id)
     pg_return_val_if_fail(panel_client != NULL, NULL);
   }
 
-
-{  static volatile int stop_here = 0;
- while(stop_here);}
-
   applet_servant->pspot = GNOME_Panel_add_applet(panel_client, applet_obj,
 						 (char *)goad_id,
 						 &privcfg,&globcfg,
@@ -701,7 +703,7 @@ applet_widget_new(const char *goad_id)
 
 	applet = APPLET_WIDGET (gtk_type_new (applet_widget_get_type ()));
 
-	CD(applet) = corbadat = gnome_panel_applet_corba_init(applet,goad_id);
+	applet->corbadat = corbadat = gnome_panel_applet_corba_init(applet,goad_id);
 
 	g_return_val_if_fail(corbadat!=NULL,NULL);
 
@@ -825,7 +827,7 @@ applet_widget_set_tooltip(AppletWidget *applet, char *text)
 	CORBA_exception_free(&ev);
 }
 
-/* Get the oprientation the applet should use */
+/* Get the orientation the applet should use */
 GNOME_Panel_OrientType
 applet_widget_get_panel_orient(AppletWidget *applet)
 {
@@ -857,12 +859,23 @@ applet_widget_init(const char *app_id,
 	/*this is not called for shlib applets so we set it to true here*/
 	die_on_last = TRUE;
 
-	gnome_client_disable_master_connection ();
 	CORBA_exception_init(&ev);
 	orb = gnome_CORBA_init_with_popt_table(app_id, app_version, 
 					       &argc, argv,
 					       options, flags, return_ctx,
 					       GNORBA_INIT_SERVER_FUNC, &ev);
+	
+	/* We need to connect to the session manager so that the session
+	 * manager waits for us to exit before exiting.
+	 *
+	 * We don't connect to the "die" signal - we expect to be told
+	 * to die by our parent. Perhaps this is bad...
+	 */
+#if 0
+	gtk_signal_connect (GTK_OBJECT (gnome_master_client ()), "die",
+			    GTK_SIGNAL_FUNC (gtk_main_quit), NULL);
+#endif	
+	gnome_client_set_restart_style (gnome_master_client(), GNOME_RESTART_NEVER);
 
 	CORBA_exception_free(&ev);
 
@@ -974,6 +987,7 @@ server_applet_back_change(CustomAppletServant *servant,
     pptr = backing->_u.pmap;
     break;
   default:
+    break;
   }
 
   gtk_signal_emit(GTK_OBJECT(servant->appwidget),
@@ -1095,8 +1109,8 @@ server_applet_factory_create_object(AppletFactory *servant,
 
 static POA_GNOME_GenericFactory__epv applet_factory_epv = {
   NULL,
-  (gpointer)&server_applet_factory_supports,
-  (gpointer)&server_applet_factory_create_object,  
+  &server_applet_factory_supports,
+  &server_applet_factory_create_object,  
 };
 
 static POA_GNOME_GenericFactory__vepv applet_factory_vepv = {
